@@ -15,7 +15,7 @@ from typing import (
 )
 
 from ....utils.async_cache import AsyncSimpleLRUCache
-from ....utils.async_tools import async_event, threaded
+from ....utils.async_tools import async_event
 from ....utils.logging import LoggingDescriptor
 from ....utils.uri import Uri
 from ...common.decorators import language_id
@@ -77,7 +77,6 @@ class RobotReferencesProtocolPart(RobotLanguageServerProtocolPart, ModelHelperMi
         return None
 
     @language_id("robotframework")
-    @threaded()
     @_logger.call
     async def collect(
         self, sender: Any, document: TextDocument, position: Position, context: ReferenceContext
